from prospector.adaptor.base import AdaptorBase


class DjangoAdaptor(AdaptorBase):
    name = 'django'

    def adapt_pylint(self, linter):
        linter.load_plugin_modules(['pylint_django'])


class CeleryAdaptor(AdaptorBase):
    name = 'celery'

    def adapt_pylint(self, linter):
        linter.load_plugin_modules(['pylint_celery'])



from __future__ import absolute_import
import json


def format_messages(summary, messages, indent=2):
    output = {}

    if summary is not None:
        output['summary'] = summary

    if messages is not None:
        output['messages'] = [m.as_dict() for m in messages]

    print json.dumps(output, indent=indent)



from prospector.adaptor.base import AdaptorBase


class CommonAdaptor(AdaptorBase):
    name = 'common-plugin'

    def adapt_pylint(self, linter):
        linter.load_plugin_modules(['pylint_common'])



import os
import re
from prospector.adaptor import LIBRARY_ADAPTORS
from requirements_detector import find_requirements
from requirements_detector.detect import RequirementsNotFound


# see http://docs.python.org/2/reference/lexical_analysis.html#identifiers
_FROM_IMPORT_REGEX = re.compile(r'^\s*from ([\._a-zA-Z0-9]+) import .*$')
_IMPORT_REGEX = re.compile(r'^\s*import ([\._a-zA-Z0-9]+)$')


def find_from_imports(file_contents):
    names = set()
    for line in file_contents.split('\n'):
        m = _IMPORT_REGEX.match(line)
        if m is None:
            m = _FROM_IMPORT_REGEX.match(line)
        if m is None:
            continue
        import_names = m.group(1).split('.')
        for import_name in import_names:
            if import_name in LIBRARY_ADAPTORS:
                names.add(import_name)
    return names


def find_from_path(path):
    names = set()
    max_possible = len(LIBRARY_ADAPTORS.keys())

    for item in os.listdir(path):
        item_path = os.path.abspath(os.path.join(path, item))
        if os.path.isdir(item_path):
            names |= find_from_path(item_path)
        elif not os.path.islink(item_path) and item_path.endswith('.py'):
            with open(item_path) as f:
                names |= find_from_imports(f.read())

        if len(names) == max_possible:
            # don't continue on recursing, there's no point!
            break

    return names


def find_from_requirements(path):
    reqs = find_requirements(path)
    names = []
    for requirement in reqs:
        if requirement.name is not None and requirement.name.lower() in LIBRARY_ADAPTORS:
            names.append(requirement.name.lower())
    return names


def autodetect_libraries(path):

    adaptor_names = []

    try:
        adaptor_names = find_from_requirements(path)
    except RequirementsNotFound:
        pass

    if len(adaptor_names) == 0:
        adaptor_names = find_from_path(path)

    adaptors = []
    for adaptor_name in adaptor_names:
        adaptors.append((adaptor_name, LIBRARY_ADAPTORS[adaptor_name]()))

    return adaptors




# This class is only referenced once, causing warning R0922, however it will be used
# more in the future as additional tools are added to Prospector
class ToolBase(object):  # pylint: disable=R0922

    def prepare(self, ignores, rootpath, args, adaptors):
        pass

    def run(self):
        raise NotImplementedError



import sys

_SUMMARY_TEMPLATE = """Started: %(started)s
Finished: %(completed)s
Time taken: %(time_taken)s seconds
Formatter: %(formatter)s
Strictness: %(strictness)s
Libraries Used: %(libraries)s
Tools run: %(tools)s
Adaptors: %(adaptors)s
Messages Found: %(message_count)d
"""


_MESSAGE_TEMPLATE = """%(module)s (%(path)s):
    L%(line)s:%(character)s %(function)s: %(source)s - %(code)s
    %(message)s
"""


def format_messages(summary, messages):
    if summary is not None:
        sys.stdout.write("Check Information\n=================\n")
        sys.stdout.write(_SUMMARY_TEMPLATE % summary)
        sys.stdout.write('\n\n')

    if messages is not None:
        sys.stdout.write("Messages\n========\n\n")
        for message in messages:
            info = {}
            info.update(message.as_dict())
            del info['location']
            info.update(message.location.as_dict())
            line = _MESSAGE_TEMPLATE % info
            sys.stdout.write(line)
            sys.stdout.write('\n')



import sys
import argparse
import os
import re
from datetime import datetime
from prospector.adaptor import LIBRARY_ADAPTORS
from prospector.adaptor.common import CommonAdaptor
from prospector.adaptor.profile import ProfileAdaptor
from prospector.autodetect import autodetect_libraries
from prospector.formatters import FORMATTERS
from prospector import tools
from prospector import __pkginfo__


def make_arg_parser():
    version = __pkginfo__.get_version()
    parser = argparse.ArgumentParser(description="Version %s. Performs analysis of Python code" % version)

    parser.add_argument('-A', '--no-autodetect', action='store_true', default=False,
                        help='Turn off auto-detection of frameworks and libraries used. By default, autodetection'
                             ' will be used. To specify manually, see the --uses option.')

    parser.add_argument('-D', '--doc-warnings', action='store_true', default=False,
                        help="Include warnings about documentation.")

    parser.add_argument('-M', '--messages-only', default=False, action='store_true',
                        help="Only output message information (don't output summary information about the checks)")

    output_help = "The output format. Valid values are %s" % ', '.join(FORMATTERS.keys())
    parser.add_argument('-o', '--output-format', default='text', help=output_help)

    parser.add_argument('--path', help="The path to the python project to inspect (defaults to PWD)")

    profiles_help = "The list of profiles to load. A profile is a certain 'type' of behaviour for prospector, " \
                    "and is represented by a YAML configuration file. A full path to the YAML file describing the " \
                    "profile must be provided." \
                    "(see --strictness)."
    parser.add_argument('-P', '--profiles', default=[], nargs='+', help=profiles_help)

    strictness_help = 'How strict the checker should be. This affects how harshly the checker will enforce' \
                      ' coding guidelines. The default value is "medium", possible values are "veryhigh", "high",' \
                      ' "medium", "low" and "verylow".'
    parser.add_argument('-s', '--strictness', help=strictness_help, default='medium')

    parser.add_argument('-S', '--summary-only', default=False, action='store_true',
                        help="Only output summary information about the checks (don't output message information)")

    tools_help = 'A list of tools to run. Possible values are: %s. By default, the following tools will be ' \
                 'run: %s' % (', '.join(tools.TOOLS.keys()), ', '.join(tools.DEFAULT_TOOLS))
    parser.add_argument('-t', '--tools', default=None, nargs='+', help=tools_help)

    parser.add_argument('-T', '--test-warnings', default=False, action='store_true',
                        help="Also check test modules and packages")

    uses_help = 'A list of one or more libraries or frameworks that the project users. Possible' \
                ' values are django, celery. This will be autodetected by default, but if autotectection' \
                ' doesn\'t work, manually specify them using this flag.'
    parser.add_argument('-u', '--uses', help=uses_help, default=[], nargs='+')

    parser.add_argument('-v', '--version', help="Print version information and exit", action='store_true')

    parser.add_argument('--absolute-paths', action='store_true', default=False,
                        help='Whether to output absolute paths when referencing files in messages. By default, '
                             'paths will be relative to the --path value')
    parser.add_argument('--no-common-plugin', action='store_true', default=False)

    return parser


def _die(message):
    sys.stderr.write('%s\n' % message)
    sys.exit(1)


def run():
    parser = make_arg_parser()
    args = parser.parse_args()

    if args.version:
        sys.stdout.write("Prospector version %s\n" % __pkginfo__.get_version())
        sys.exit(0)

    summary = {
        'started': datetime.now()
    }

    path = args.path or os.getcwd()

    try:
        formatter = FORMATTERS[args.output_format]
        summary['formatter'] = args.output_format
    except KeyError:
        _die("Formatter %s is not valid - possible values are %s" % (args.output_format, ', '.join(FORMATTERS.keys())))

    libraries_used = []
    profiles = []
    adaptors = []

    if not args.no_common_plugin:
        adaptors.append(CommonAdaptor())
    if not args.no_autodetect:
        for libname, adaptor in autodetect_libraries(path):
            libraries_used.append(libname)
            adaptors.append(adaptor)

    strictness = args.strictness
    strictness_options = ('veryhigh', 'high', 'medium', 'low', 'verylow')
    if strictness not in strictness_options:
        possible = ', '.join(strictness_options)
        _die("%s is not a valid value for strictness - possible values are %s" % (strictness, possible))
    else:
        profiles.append('strictness_%s' % strictness)
        summary['strictness'] = strictness

    for library in args.uses:
        if library not in LIBRARY_ADAPTORS:
            possible = ', '.join(LIBRARY_ADAPTORS.keys())
            _die("Library/framework %s is not valid - possible values are %s" % (library, possible))
        libraries_used.append(library)
        adaptors.append(LIBRARY_ADAPTORS[library]())

    summary['libraries'] = ', '.join(libraries_used)

    if not args.doc_warnings:
        profiles.append('no_doc_warnings')

    if not args.test_warnings:
        profiles.append('no_test_warnings')

    profiles += args.profiles

    profile_adaptor = ProfileAdaptor(profiles)
    adaptors.append(profile_adaptor)

    summary['adaptors'] = []
    for adaptor in adaptors:
        summary['adaptors'].append(adaptor.name)
    summary['adaptors'] = ', '.join(summary['adaptors'])

    tool_runners = []
    tool_names = args.tools or tools.DEFAULT_TOOLS
    for tool in tool_names:
        if not tool in tools.TOOLS:
            _die("Tool %s is not valid - possible values are %s" % (tool, ', '.join(tools.TOOLS.keys())))
        tool_runners.append(tools.TOOLS[tool]())

    summary['tools'] = ', '.join(tool_names)

    ignore = map(re.compile, profile_adaptor.profile.ignore)
    for tool in tool_runners:
        tool.prepare(path, ignore, args, adaptors)

    messages = []
    for tool in tool_runners:
        messages += tool.run()

    for message in messages:
        if args.absolute_paths:
            message.to_absolute_path(path)
        else:
            message.to_relative_path(path)

    summary['message_count'] = len(messages)
    summary['completed'] = datetime.now()
    delta = (summary['completed'] - summary['started'])
    summary['time_taken'] = '%0.2f' % delta.total_seconds()

    summary['started'] = str(summary['started'])
    summary['completed'] = str(summary['completed'])

    if args.messages_only:
        summary = None
    if args.summary_only:
        messages = None

    formatter(summary, messages)

    sys.exit(0)


if __name__ == '__main__':
    run()



import sys
import os
from prospector.tools.base import ToolBase
from prospector.tools.pylint.collector import Collector
from prospector.tools.pylint.linter import ProspectorLinter


def _find_package_paths(ignore, rootpath):
    sys_path = set()
    check_dirs = []

    for subdir in os.listdir(rootpath):
        if subdir.startswith('.'):
            continue

        subdir_fullpath = os.path.join(rootpath, subdir)
        rel_path = os.path.relpath(subdir_fullpath, rootpath)

        if os.path.islink(subdir_fullpath):
            continue

        if os.path.isfile(subdir_fullpath):
            if not subdir.endswith('.py'):         
                continue

            if os.path.exists(os.path.join(rootpath, '__init__.py')):
                continue

            # this is a python module but not in a package, so add it
            if any([m.search(rel_path) for m in ignore]):
                continue
            check_dirs.append(subdir_fullpath)
         
        elif os.path.exists(os.path.join(subdir_fullpath, '__init__.py')):
            # this is a package, add it and move on
            if any([m.search(rel_path) for m in ignore]):
                continue
            sys_path.add(rootpath)
            check_dirs.append(subdir_fullpath)
        else:
            # this is not a package, so check its subdirs
            add_sys_path, add_check_dirs = _find_package_paths(ignore, subdir_fullpath)
            sys_path |= add_sys_path
            check_dirs += add_check_dirs
    return sys_path, check_dirs


class PylintTool(ToolBase):

    def __init__(self):
        self._args = self._extra_sys_path = self._collector = self._linter = None

    def prepare(self, rootpath, ignore, args, adaptors):
        linter = ProspectorLinter(ignore)
        linter.load_default_plugins()

        extra_sys_path, check_paths = _find_package_paths(ignore, rootpath)

        # insert the target path into the system path to get correct behaviour
        self._orig_sys_path = sys.path
        # note: we prepend, so that modules are preferentially found in the path
        # given as an argument. This prevents problems where we are checking a module
        # which is already on sys.path before this manipulation - for example, if we
        # are checking 'requests' in a local checkout, but 'requests' is already
        # installed system wide, pylint will discover the system-wide modules first
        # if the local checkout does not appear first in the path
        sys.path = list(extra_sys_path) + sys.path

        for adaptor in adaptors:
            adaptor.adapt_pylint(linter)

        self._args = linter.load_command_line_configuration(check_paths)

        # disable the warnings about disabling warnings...
        linter.disable('I0011')
        linter.disable('I0012')
        linter.disable('I0020')
        linter.disable('I0021')

        # use the collector 'reporter' to simply gather the messages given by PyLint
        self._collector = Collector()
        linter.set_reporter(self._collector)

        self._linter = linter

    def run(self):
        # note: Pylint will exit with a status code indicating the health of the
        # code it was checking. Prospector will not mimic this behaviour, as it
        # interferes with scripts which depend on and expect the exit code of the
        # code checker to match whether the check itself was successful
        self._linter.check(self._args)
        sys.path = self._orig_sys_path

        return self._collector.get_messages()



# -*- coding: UTF-8 -*-
from distutils.core import setup
from setuptools import find_packages
import time
from prospector import __pkginfo__

_packages = find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])

_short_description = "Prospector: python static analysis tool"

_install_requires = [
    'pylint>=1.0.0',
    'pylint-celery>=0.1',
    'pylint-django>=0.1',
    'pylint-plugin-utils>=0.1',
    'pylint-common>=0.1',
    'requirements-detector>=0.1.1',
    'argparse==1.2.1',
    'pyyaml',
]

_package_data = {
    'prospector': [
        'profiles/profiles/no_doc_warnings.yaml',
        'profiles/profiles/no_test_warnings.yaml',
        'profiles/profiles/strictness_high.yaml',
        'profiles/profiles/strictness_low.yaml',
        'profiles/profiles/strictness_medium.yaml',
        'profiles/profiles/strictness_veryhigh.yaml',
        'profiles/profiles/strictness_verylow.yaml',
    ]
}

_classifiers = (
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
    'Operating System :: Unix',
    'Topic :: Software Development :: Quality Assurance',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
)

setup(
    name='prospector',
    url='https://github.com/landscapeio/prospector',
    author='landscape.io',
    author_email='code@landscape.io',
    description=_short_description,
    install_requires=_install_requires,
    package_data=_package_data,
    scripts=['bin/prospector'],
    version=__pkginfo__.get_version(),
    packages=_packages,
    license='GPLv2',
    keywords='pylint prospector code analysis',
    classifiers=_classifiers
)



import os
import yaml


class ProfileNotFound(Exception):
    def __init__(self, name, filepath):
        self.name = name
        self.filepath = filepath

    def __repr__(self):
        return "Could not find profile %s at %s" % (self.name, self.filepath)


_empty_data = {
    'inherits': [],
    'pylint': {
        'disable': [],
        'options': {}
    },
    'ignore': [],
}


def load_profiles(names, basedir=None):
    if not isinstance(names, (list, tuple)):
        names = (names,)
    profiles = [_load_profile(name, basedir=basedir)[0] for name in names]
    return merge_profiles(profiles)


def _load_content(name, basedir=None):

    if name.endswith('.yaml'):
        # assume that this is a full path that we can load
        filename = name
    else:
        basedir = basedir or os.path.join(os.path.dirname(__file__), 'profiles')
        filename = os.path.join(basedir, '%s.yaml' % name)

    if not os.path.exists(filename):
        raise ProfileNotFound(name, os.path.abspath(filename))

    with open(filename) as f:
        return f.read()


def from_file(name, basedir=None):
    return parse_profile(name, _load_content(name, basedir))


def _load_profile(name, basedir=None, inherits_set=None):
    inherits_set = inherits_set or set()

    profile = parse_profile(name, _load_content(name, basedir))
    inherits_set.add(profile.name)

    for inheritsed in profile.inherits:
        if inheritsed not in inherits_set:
            inheritsed_profile, sub_inherits_set = _load_profile(inheritsed, basedir, inherits_set)
            profile.merge(inheritsed_profile)
            inherits_set |= sub_inherits_set

    return profile, inherits_set


def parse_profile(name, contents):
    if name.endswith('.yaml'):
        # this was a full path
        name = os.path.splitext(os.path.basename(name))[0]
    data = yaml.load(contents)
    if data is None:
        # this happens if a completely empty YAML file is passed in to parse_profile, for example
        data = dict(_empty_data)
    else:
        data = _merge_dict(_empty_data, data, d1_priority=False)
    return StrictnessProfile(name, data)


def _merge_dict(d1, d2, dedup_lists=False, d1_priority=True):
    newdict = {}
    newdict.update(d1)

    for key, value in d2.iteritems():
        if key not in d1:
            newdict[key] = value
        elif value is None and d1[key] is not None:
            newdict[key] = d1[key]
        elif d1[key] is None and value is not None:
            newdict[key] = value
        elif type(value) != type(d1[key]):
            raise ValueError("Could not merge conflicting types %s and %s" % (type(value), type(d1[key])))
        elif isinstance(value, dict):
            newdict[key] = _merge_dict(d1[key], value, dedup_lists, d1_priority)
        elif isinstance(value, (list, tuple)):
            newdict[key] = list(set(d1[key]) | set(value))
        elif not d1_priority:
            newdict[key] = value

    return newdict


class StrictnessProfile(object):

    def __init__(self, name, profile_dict):
        self.name = name
        self.pylint = profile_dict['pylint']
        self.inherits = profile_dict['inherits']
        self.ignore = profile_dict['ignore']

    def to_profile_dict(self):
        return {
            'inherits': self.inherits,
            'ignore': self.ignore,
            'pylint': self.pylint
        }

    def merge(self, other_profile):
        self.ignore = list(set(self.ignore + other_profile.ignore))
        self.inherits = list(set(self.inherits + other_profile.inherits))
        self.pylint = _merge_dict(self.pylint, other_profile.pylint)


def merge_profiles(profiles):
    merged_profile = profiles[0]
    for profile in profiles[1:]:
        merged_profile.merge(profile)
    return merged_profile



import os


class Location(object):

    def __init__(self, path, module, function, line, character):
        self.path = path
        self._path_is_absolute = True
        self.module = module
        self.function = function
        self.line = line
        self.character = character

    def to_absolute_path(self, root):
        if self._path_is_absolute:
            return
        self.path = os.path.abspath(os.path.join(root, self.path))
        self._path_is_absolute = True

    def to_relative_path(self, root):
        if not self._path_is_absolute:
            return
        self.path = os.path.relpath(self.path, root)
        self._path_is_absolute = False

    def as_dict(self):
        return {
            'path': self.path,
            'module': self.module,
            'function': self.function,
            'line': self.line,
            'character': self.character
        }


class Message(object):

    def __init__(self, source, code, location, message):
        self.source = source
        self.code = code
        self.location = location
        self.message = message

    def to_absolute_path(self, root):
        self.location.to_absolute_path(root)

    def to_relative_path(self, root):
        self.location.to_relative_path(root)

    def as_dict(self):
        return {
            'source': self.source,
            'code': self.code,
            'location': self.location.as_dict(),
            'message': self.message
        }



from prospector.adaptor.base import AdaptorBase
from prospector.profiles.profile import load_profiles
from pylint.utils import UnknownMessage


class ProfileAdaptor(AdaptorBase):

    def __init__(self, profile_names):
        self.profile = load_profiles(profile_names)
        self.name = 'profiles:%s' % ','.join(profile_names)

    def adapt_pylint(self, linter):
        for msg_id in self.profile.pylint['disable']:
            try:
                linter.disable(msg_id)
            except UnknownMessage:
                pass

        options = self.profile.pylint['options']

        for checker in linter.get_checkers():
            if not hasattr(checker, 'options'):
                continue
            for option in checker.options:
                if option[0] in options:
                    checker.set_option(option[0], options[option[0]])
